//
//  UserImageDetailViewController.swift
//  PragmaticTweets
//
//  Created by Chris Adamson on 10/6/15.
//  Copyright © 2015 Pragmatic Programmers, LLC. All rights reserved.
//

import UIKit

class UserImageDetailViewController: UIViewController {
	
	var userImageURL: NSURL?
  @IBOutlet weak var userImageView: UIImageView!
	var preGestureTransform: CGAffineTransform?
  
  override func viewWillAppear(animated: Bool) {
    super.viewWillAppear(animated)
    if let userImageURL = userImageURL,
      imageData = NSData (contentsOfURL: userImageURL) {
        userImageView.image = UIImage(data: imageData)
    }
  }
	
  @IBAction func handlePanGesture(sender: UIPanGestureRecognizer) {
    if sender.state == .Began {
      preGestureTransform = userImageView.transform
    }
    if sender.state == .Began ||
      sender.state == .Changed {
        let translation = sender.translationInView(userImageView) 
        let translatedTransform = CGAffineTransformTranslate( 
          preGestureTransform!, translation.x, translation.y) 
        userImageView.transform = translatedTransform 
    }
  }
  
  @IBAction func handleDoubleTapGesture(sender: UITapGestureRecognizer) {
    userImageView.transform = CGAffineTransformIdentity
  }
  
  @IBAction func handlePinchGesture(sender: UIPinchGestureRecognizer) {
    if sender.state == .Began { 
      preGestureTransform = userImageView.transform
    } 
    if sender.state == .Began || 
      sender.state == .Changed { 
        let scaledTransform = CGAffineTransformScale( 
          preGestureTransform!, sender.scale, sender.scale) 
        userImageView.transform = scaledTransform 
    }
  }
	
}
