//: Playground - noun: a place where people can play
import UIKit
import AVFoundation
import XCPlayground
XCPlaygroundPage.currentPage.needsIndefiniteExecution = true

/* first version cited in book
class IOSDevice {
	var name : String
	var screenHeight : Double
	var screenWidth : Double
}
*/

/* first version with an initializer
class IOSDevice {
  var name : String
  var screenHeight : Double
  var screenWidth : Double

  init (name: String, screenHeight: Double, screenWidth: Double) { 
    self.name = name
    self.screenHeight = screenHeight
    self.screenWidth = screenWidth
  } 
}
*/


class IOSDevice : CustomStringConvertible {
  var name : String
  var screenHeight : Double
  var screenWidth : Double

  private var audioPlayer : AVPlayer?
	
  init (name: String, screenHeight: Double, screenWidth: Double) {
    self.name = name
    self.screenHeight = screenHeight
    self.screenWidth = screenWidth
  }

  var description: String {
    return "\(name), \(screenHeight) x \(screenWidth)"
  }

  var screenArea : Double {
    get {
     return screenWidth * screenHeight
    }
  }

  // for tuple section
  var screenHeightAndWidth : (height: Double, width: Double) {
    get {
      return (screenHeight, screenWidth)
    }
  }
	
  func playAudioWithURL(url: NSURL) -> Void {
    audioPlayer = AVPlayer(URL: url)
    audioPlayer!.play()
  }

  func stopAudio() -> Void {
    if let audioPlayer = audioPlayer {
      audioPlayer.pause()
    }
    audioPlayer = nil
  }
	
}

// calling the class:

let iPhone6 = IOSDevice(name: "iPhone 6",
  screenHeight: 138.1, screenWidth: 67.0)

// call computed property
iPhone6.screenArea

/*
// call method
if let url = NSURL(string: "http://armitunes.com:8010/listen.pls") {
  iPhone6.playAudioWithURL(url)
}
*/


// tuples
iPhone6.screenHeightAndWidth
iPhone6.screenHeightAndWidth.height
iPhone6.screenHeightAndWidth.0

// iterate with tuple
let iPhone6Plus = IOSDevice(name: "iPhone 6 Plus",
  screenHeight: 158.1, screenWidth: 77.8)
let iPhone5s = IOSDevice (name: "iPhone 5s",
  screenHeight: 123.8, screenWidth: 58.6)
let iPhones = [iPhone5s, iPhone6, iPhone6Plus]
for (index, phone) in iPhones.enumerate() {
  NSLog ("\(index): \(phone)")
}

/* hypothetical - see how functions pass around references to their objects.
compare later to how structs, enums, etc. pass copies of the data.

func destroyIOSDevice (device: IOSDevice) {
  device.screenWidth = 0.0
  device.screenHeight = 0.0
  device.name = ""
}

destroyIOSDevice(iPhone6)
iPhone6

*/

