//: Playground - noun: a place where people can play

import UIKit

enum ScreenType {
  case None
  case Retina (screenHeight: Double, screenWidth: Double)
}


struct IOSDevice {
  var name : String
  var screenType : ScreenType
}

let iPhone6 = IOSDevice(name: "iPhone 6",
  screenType: ScreenType.Retina(screenHeight: 138.1, screenWidth: 67.0)) 
let appleTV3rdGen = IOSDevice(name: "Apple TV (3rd Gen)",
  screenType: ScreenType.None)

/* variation of appleTV3rdGen initializer that omits "ScreenType"
let appleTV3rdGen = IOSDevice(name: "Apple TV (3rd Gen)",
  screenType: .None)
*/

extension IOSDevice : CustomStringConvertible { 
  var description : String {
    var screenDescription: String 
    switch screenType { 
    case .None: 
      screenDescription = "No screen"
    case .Retina (let screenHeight, let screenWidth): 
      screenDescription = "Retina screen " +
        "\(screenHeight) x \(screenWidth)" 
    }
    return "\(name): \(screenDescription)" 
  }
}

/* not sure if we'll use this?
extension IOSDevice : Equatable {
}
*/

// operator overloading!
func == (lhs: IOSDevice, rhs: IOSDevice) -> Bool {
  if lhs.name != rhs.name {
    return false
  }
  
  switch lhs.screenType {
  case .None:
    switch rhs.screenType {
      case .None:
        return true
    default:
      return false
    }
  case .Retina(let lhsScreenHeight, let lhsScreenWidth):
    switch rhs.screenType {
    case .None:
      return false
    case .Retina(let rhsScreenHeight, let rhsScreenWidth):
      return lhsScreenHeight == rhsScreenHeight &&
        lhsScreenWidth == rhsScreenWidth
    }
  }
}

// note: be sure to add that you can use "_" for associated values you don't care about

iPhone6 == iPhone6
appleTV3rdGen == iPhone6


// optionals demo
let optionalString : String? = "iPhone6"
switch optionalString {
case .None:
  NSLog ("nil!")
case .Some(let value):
  NSLog ("some! \(value)")
}
